IOC_ARGUMENTS_NAME = '__ioc_arguments__'
LIFECYCLE_ARGUMENT_NAME = '__lifecycle__'


def arguments(**kwargs):
    def real_decorator(func):
        setattr(func, IOC_ARGUMENTS_NAME, kwargs)
        return func

    return real_decorator


class LifeCycle(object):
    SINGLETON = 0
    MULTI_INSTANCE = 1


def life_cycle(life_cycle_code: int):
    def real_decorator(func):
        setattr(func, LIFECYCLE_ARGUMENT_NAME, life_cycle_code)
        return func

    return real_decorator



from typing import Type


class ContainerException(Exception):
    def __init__(self, cls: Type, message):
        self.cls = cls
        super(ContainerException, self).__init__(message)


class ClassNotFoundException(ContainerException):
    def __init__(self, cls: Type):
        super(ClassNotFoundException, self).__init__(cls,
                                                     'Could not find registered implementation for class: "{}"'.format(
                                                         cls.__name__))


class ConfigProviderRequiredException(ContainerException):
    def __init__(self, cls: Type, argument_name):
        self.cls = cls
        self.argument = argument_name
        super(ConfigProviderRequiredException, self).__init__(cls,
                                                              'The argument "{}" requires config provider for class '
                                                              '"{}" and you did not configure one'.format(argument_name,
                                                                                                          cls.__name__))
